Machine learning for Premise Selection
======================================


Introduction
------------

This document describes the installation and usage of `predict`, a suite
of premise selection tools. The purpose of premise selection is to suggest
premises for conjectures which maximise an ATP's chances of proving the
conjecture, based on information which premises were useful in previous proofs.

`predict` currently implements several machine learning techniques:

* k-nearest neighbours
* Naive Bayes
* Random forests

This package contains three programs:

* `predict`: select premises based on machine learning
* `statistics`: evaluate prediction quality by comparing with actual premises
* `combine`: combine different predictions


Installation
------------

`predict` requires the Haskell compiler `ghc` (tested with version 7.8.4)
and the build tool `stack`.

You can build the package via:

    stack install


File formats
------------

For each dataset containing information about previous proofs, `predict`
needs at least three files:

- Features: Gives features (e.g. symbols) for each theorem.
  An example line is:

        ADD_SYM:"+", "=", "forall", "num", "bool"

- Dependencies: Gives the dependencies of each theorem, i.e. the premises that
  were used to prove it. An example line is:

        ADD_SYM:ADD_CLAUSES ADD ADD_SUC REFL_CLAUSE FORALL_SIMP num_INDUCTION

- Sequence: Gives the sequence in which theorems should be learned.
  Example lines are:

        ADD
        ADD_SUC
        ADD_CLAUSES
        ADD_SYM

- Evaluation: Optionally, one can supply an evaluation file to the predictor,
  which contains a list of theorems for which advice will be output. Example:

        ADD_SUC
        ADD_SYM


`predict` ships with three datasets, namely from following proof assistants:

* HOL Light
* Mizar
* Isabelle


Prediction
----------

If no evaluation file is given, the predictor learns all data and then waits
for a list of features on standard input, for which it will then output
appropriate theorems sorted by decreasing relevance.

To not always have to supply features, dependencies, and sequence files to the
predictor for a dataset, one can define shortcut commands for `predict`,
such as `predictmizar.sh`. Usage examples:

- Load the evaluation file "mizar/eval" and run a random forest predictor with
  4 trees and a sample frequency of 2:

        ./predictisabelle.sh -e isabelle/eval -p rforest:nTrees=4:sampleFreq=2

- Learn HOL Light data with a k-NN predictor, ignore IDF information and give
  a dependency weight of 1.5. Output maximally 256 premises relevant to prove
  conjectures having the feature `A0bool`:

        echo "\"A0bool\"" | ./predicthl.sh -p knn:idf=False:depWeight=1.5 -n 256

- Train Naive Bayes predictor on Isabelle data, and show statistics (see below):

        ./predictmizar.sh -e mizar/eval -p nbayes | ./statisticsmizar.sh

Predictor options are separated by ":", and are documented in the respective
source files.


Statistics
----------

To evaluate the prediction quality, one can compare the predictions for a
theorem with its actual dependencies (the premises it was proved with). For
this, one can use the `statistics` program, again available for multiple
datasets via e.g. `statisticsmizar.sh`. It reads predictions either from
standard input or from a list of files. For example:

    ./statisticsmizar.sh mz_knn mz_rforest


Combining predictions
---------------------

Combining the outputs of different predictors can give advantages over
using the predictions of a single predictor only: `combine` accepts multiple
prediction files together with weights, which will determine the relative
impact of the predictions in each individual file. It outputs the combined
predictions on stdout.

For example, to combine k-NN and Naive Bayes prediction files for HOL Light
using the harmonic mean, where k-NN gets weight 0.3 and Naive Bayes weight 0.7:

    ./combine.sh hl/seq hl/eval Harmonic 1024 hl_knn 0.3 hl_nbayes 0.7


Experiments
-----------

To run a large number of experiments, we use `shake`, a Haskell-based build
system similar to `make`.

    experiments --report -j4

This will run four experiments in parallel, creating a "report.html" that
contains information how long each experiment took.

