#!/usr/bin/env runhaskell

import Control.Applicative ((<$>))
import Data.List.Split (splitOneOf)
import Options.Applicative


treatEvalLine deps line =
  let es = firstInt line
      ds = takeWhile (\ l -> firstInt l /= es) deps
  in (es, unwords ("1" : (tail $ words line)) ++ "\n", ds)

  where firstInt = head . splitOneOf " ,"

data Files = Files {
  evaluationFile :: FilePath
, labelsFile   :: FilePath
}

filesParser:: Parser Files
filesParser = Files
  <$> argument str (metavar "EVALUATION")
  <*> argument str (metavar "LABELS")

parserInfo :: ParserInfo Files
parserInfo = info (helper <*> filesParser)
  ( fullDesc
  <> progDesc "Create iterative ML problems from LibSVM files")

main = do
  Files evalFP depsFP <- execParser parserInfo

  evalF <- lines <$> readFile evalFP
  depsF <- lines <$> readFile depsFP

  let ed = map (treatEvalLine depsF) evalF
  mapM_ (\ (f, e, d) ->
    writeFile (evalFP ++ "-" ++ f) e >>
    writeFile (depsFP ++ "-" ++ f) (unlines d)) ed
