module LibSVM where

import Data.Foldable (toList)
import Data.List (intercalate, sort)
import qualified Data.Sequence as Seq

import Options.Applicative

import qualified Interpreter as I
import Types


svmLine :: Labels -> Double -> Features -> String
svmLine lbls c = unwords . (showLabels lbls :) . map showFeature where
  showLabels = intercalate "," . map show
  showFeature f = show f ++ ":" ++ show c

svmLines :: [(SampleID, Labels, Features)] -> [String]
svmLines = map (\ (SampleID smpl, lbls, ftrs) -> svmLine (smpl:lbls) 1.0 ftrs)
{-
svmLines lblsL ftrsL =
  (map (\ (lbls, ftrs) -> svmLine  lbls  1.0 ftrs) $ filter (not . null . fst) $ zip lblsL   ftrsL) ++
  (map (\ (smpl, ftrs) -> svmLine [smpl] 0.8 ftrs) $ zip [0 ..] ftrsL)
-}


data Files = Files {
  featuresFile :: FilePath
, labelsFile   :: FilePath
, sequenceFile :: FilePath
, evaluationFile :: Maybe FilePath
}

filesParser:: Parser Files
filesParser = Files
  <$> argument str (metavar "FEATURES")
  <*> argument str (metavar "LABELS")
  <*> argument str (metavar "SEQUENCE")
  <*> optional (argument str (metavar "EVALUATION"))

parserInfo :: ParserInfo Files
parserInfo = info (helper <*> filesParser)
  ( fullDesc
  <> progDesc "Convert premise selection data to LibSVM format")

loadEvalMaybe :: LabelBM -> Maybe FilePath -> IO (Samples, Seq.Seq a -> [a])
loadEvalMaybe lblMap = maybe (return (map SampleID [0 ..], toList))
    (\ fp -> I.loadEvaluationFile lblMap fp >>=
     \ eval -> return (eval, \ s -> map (\ (SampleID e) -> Seq.index s e) eval))

main :: IO ()
main = do
  Files ftrsF depsF seqF evalF <- execParser parserInfo

  lblMap            <- I.loadSequenceFile seqF
  SampleLabels lbls <- I.loadLabelsFile lblMap depsF
  (ftrs, _)    <- I.loadFeaturesFile lblMap ftrsF
  (smpls, transform) <- loadEvalMaybe lblMap evalF

  let lblsL = map sort $ transform lbls
      ftrsL = map sort $ transform ftrs
  
  mapM_ putStrLn $ svmLines $ zip3 smpls lblsL ftrsL
