{-# LANGUAGE GeneralizedNewtypeDeriving #-}

module Types where

import           Control.DeepSeq (NFData)
import           Data.ByteString (ByteString)
import           Data.IntMap (IntMap)
import           Data.Sequence (Seq)

import           IntBiMap (IntBiMap)


-- -----------------------------------------------------------------------------
-- Type definitions

newtype  SampleID = SampleID Int deriving (Eq, NFData, Ord, Show)
type   LabelID = Int
type FeatureID = Int

type  Samples = [ SampleID]
type   Labels = [  LabelID]
type Features = [FeatureID]

newtype SampleFeatures = SampleFeatures (Seq Features)
newtype   SampleLabels =   SampleLabels (Seq   Labels)

-- | Map between feature IDs and their names.
type   FeatureBM = IntBiMap ByteString
type     LabelBM = IntBiMap ByteString

type  SampleMap = IntMap
type   LabelMap = IntMap
type FeatureMap = IntMap

type FeatureFreq = FeatureMap Int
type   LabelFreq =   LabelMap Int

data LearnData = LearnData {
  sampleLabels   :: SampleLabels
, sampleFeatures :: SampleFeatures
, labelBM        :: LabelBM
, featureBM      :: FeatureBM
, nPredictions   :: Int
}
